/////////////////////////////////////////////////////////////////////////
////                          EX_PBUSR.C                             ////
////                                                                 ////
////  This program shows a simple chip to chip sharred (shadow) RAM  ////
////  using a one wire interface between the chips. Any (reasonable) ////
////  number of chips may be connected to the one wire.  The program ////
////  also shows how to handle two serial ports within one program.  ////
////                                                                 ////
/////////////////////////////////////////////////////////////////////////
////        (C) Copyright 1996,1997 Custom Computer Services            ////
//// This source code may only be used by licensed users of the CCS C   ////
//// compiler.  This source code may only be distributed to other       ////
//// licensed users of the CCS C compiler.  No other use, reproduction  ////
//// or distribution is permitted without written permission.           ////
//// Derivative programs created using this software in object code     ////
//// form are not restricted in any way.                                ////
////////////////////////////////////////////////////////////////////////////

#include <SX52BD.H>
#include "atd_emu.h"

#fuses HS,NOWDT,NOPROTECT

#use standard_io(B)
#use delay(clock=16000000)
#use rs232(baud=9600,float_high,bits=9,xmit=PIN_B0,rcv=PIN_B0)

#define PBUS_RAM_SIZE 16

byte pbus_ram[PBUS_RAM_SIZE];
enum pbus_states {PBUS_IDLE=0x80,PBUS_NEED_CS=0x84};
byte pbus_state=PBUS_IDLE;
byte pbus_next_loc,pbus_next_value;
short pbus_ram_changed;

#bit ninth_bit = RS232_ERRORS.7
#bit collision = RS232_ERRORS.6
#bit intf = 11.1

#int_rb0
void pbus_isr() {
    byte data;

    if(kbhit()) {

      data=getc();

      if(ninth_bit) {
           if(data==0xf2)
              pbus_state=0;
      } else
        if(pbus_state==PBUS_NEED_CS) {
           if(data==(pbus_next_loc^pbus_next_value)) {
               pbus_state=PBUS_IDLE;
               pbus_ram_changed=true;
               pbus_ram[pbus_next_loc]=pbus_next_value;
           }
        } else if(pbus_state==0) {
           pbus_next_loc=data;
           pbus_state=1;
        } else if(pbus_state==1) {
           pbus_next_value=data;
           pbus_state=PBUS_NEED_CS;
      }
    }
}

void pbus_write_ram(byte loc, byte value) {
   byte checksum,i;

   pbus_ram_changed=true;
   retry:
      checksum=loc^value;
      setup_miwu (miwuOFF, miwuOFF, miwuOFF, miwuOFF, miwuOFF, miwuOFF, miwuOFF, miwuOFF);
      pbus_ram[loc]=value;
      collision=false;
      ninth_bit=1;

      putc(0xf2);           if(collision) goto error;
      ninth_bit=0;
      putc(loc);            if(collision) goto error;
      putc(value);          if(collision) goto error;
      putc(checksum);       if(collision) goto error;
      intf=false;
      setup_miwu (miwuOFF, miwuOFF, miwuOFF, miwuOFF, miwuOFF, miwuOFF, miwuOFF, miwuH2L);
      return;

   error:
      delay_ms(16);
      setup_miwu (miwuOFF, miwuOFF, miwuOFF, miwuOFF, miwuOFF, miwuOFF, miwuOFF, miwuH2L);
      goto retry;
}

#use rs232(baud=19200, xmit=PIN_A3, rcv=PIN_A2)
#include <input.c>

main() {
   byte to,i,value;

   set_mode_a(mTTLIN,mTTLIN,mTTLIN,mTTLIN,mOUTPUT,mTTLIN,mTTLIN,mTTLIN);
   setup_miwu (miwuOFF, miwuOFF, miwuOFF, miwuOFF, miwuOFF, miwuOFF, miwuOFF, miwuH2L);

   printf("\r\nPress S to send.\r\n");

   pbus_ram_changed=false;

   do {
      if(pbus_ram_changed) {
         printf("\r\nRAM Changed:\r\n");
         pbus_ram_changed=false;
         for(i=0;i<PBUS_RAM_SIZE;++i)
           printf(" %2X",pbus_ram[i]);
      }

      if(kbhit())
        if(toupper(getc())=='S') {
           printf("\r\nLocation: ");
           to=gethex();
           printf("\r\nValue: ");
           value=gethex();
           pbus_write_ram(to,value);
      }

   } while (TRUE);
}
