/////////////////////////////////////////////////////////////////////////
////                          EX_SRAM.C                              ////
////                                                                 ////
////  This program uses the PCF8570.C SRAM driver to read and write  ////
////  to an external (2 wire) serial SRAM.                           ////
////                                                                 ////
////  The 68HC68R1.C or 68HC68R2.C drivers may be used in place of   ////
////  the PCF8570.C driver.                                          ////
////                                                                 ////
////  Pin layout for the 68HC68R1:                                   ////
////                                                                 ////
////  SX Chip    68HC68R1                                            ////
////  -------------------                                            ////
////  PIN_B7  --->  PIN 5                                            ////
////  PIN_B6  --->  PIN 6                                            ////
////  PIN_B5  --->  PIN 1                                            ////
////  PIN_B4  --->  PIN 7                                            ////
////  PIN_B3  --->  PIN 2                                            ////
////  GND     --->  PIN 4                                            ////
////  +5      --->  PIN 8                                            ////
////                                                                 ////
/////////////////////////////////////////////////////////////////////////
////        (C) Copyright 1996,1997 Custom Computer Services            ////
//// This source code may only be used by licensed users of the CCS C   ////
//// compiler.  This source code may only be distributed to other       ////
//// licensed users of the CCS C compiler.  No other use, reproduction  ////
//// or distribution is permitted without written permission.           ////
//// Derivative programs created using this software in object code     ////
//// form are not restricted in any way.                                ////
////////////////////////////////////////////////////////////////////////////

#include <SX28AC.H>
#include "atd_emu.h"

#fuses HS,NOWDT,NOPROTECT

#use standard_io(B)
#use delay(clock=16000000)
#use rs232(baud=19200, xmit=PIN_A3, rcv=PIN_A2)

#include <INPUT.C>

//#include <PCF8570.C>
#include <68HC68R1.C>


main() {
   byte value,cmd;
   byte address;

   set_mode_a(mTTLIN,mTTLIN,mTTLIN,mTTLIN,mOUTPUT,mTTLIN,mTTLIN,mTTLIN);

   do {
      do {
         printf("\r\nRead or Write: ");
         cmd=getc();
         cmd=toupper(cmd);
         putc(cmd);
      } while ( (cmd!='R') && (cmd!='W') );

      printf("\n\rLocation: ");

      address = gethex();

      if(cmd=='R')
         printf("\r\nValue: %X\r\n",READ_EXT_SRAM( address ) );

      if(cmd=='W') {
         printf("\r\nNew value: ");
         value = gethex();
         printf("\n\r");
         WRITE_EXT_SRAM( address, value );
      }
   } while (TRUE);

}
