/////////////////////////////////////////////////////////////////////////
////                           EX_STWT.C                             ////
////                                                                 ////
////  This program uses the RTCC (timer0) and interrupts to keep a   ////
////  real time seconds counter.  A simple stop watch function is    ////
////  then implemented.                                              ////
////                                                                 ////
/////////////////////////////////////////////////////////////////////////
////        (C) Copyright 1996,1997 Custom Computer Services            ////
//// This source code may only be used by licensed users of the CCS C   ////
//// compiler.  This source code may only be distributed to other       ////
//// licensed users of the CCS C compiler.  No other use, reproduction  ////
//// or distribution is permitted without written permission.           ////
//// Derivative programs created using this software in object code     ////
//// form are not restricted in any way.                                ////
////////////////////////////////////////////////////////////////////////////

#include <SX52BD.H>
#include "atd_emu.h"

#fuses HS,NOWDT,NOPROTECT

#use delay(clock=16000000)
#use rs232(baud=19200, xmit=PIN_A3, rcv=PIN_A2)

#define INTS_PER_SECOND 244    // (16000000/(256*256))

byte seconds;      // A running seconds counter
byte int_count;    // Number of interrupts left before a second has elapsed


#int_rtcc                          // This function is called every time
clock_isr() {                      // the RTCC (timer0) overflows (255->0).
                                   // For this program this is apx 76 times
    if(--int_count==0) {           // per second.
      ++seconds;
      int_count=INTS_PER_SECOND;
    }
}

main() {
   byte start;

   set_mode_a(mTTLIN,mTTLIN,mTTLIN,mTTLIN,mOUTPUT,mTTLIN,mTTLIN,mTTLIN);

   int_count=INTS_PER_SECOND;
   set_rtcc(0);
   setup_counters(RTCC_INT | RTCC_INTERNAL, RTCC_DIV_256);

   do {
      printf("\r\nPress any key to begin.\n\r");
      getc();
      start=seconds;
      printf("Press any key to stop.\n\r");
      getc();
      printf("%u seconds.\n\r",seconds-start);
   } while (TRUE);
}
