/////////////////////////////////////////////////////////////////////////
////                             EX_X10.C                            ////
////                                                                 ////
////  This program interfaces a X10 TW523 unit to RS-232.  This      ////
////  program will accept and send three character codes of the      ////
////  form xyy where x is A-P and yy is 00-1F.                       ////
////  Key codes 00-0F are translated to the key number.              ////
////                                                                 ////
////  A * is sent to indicate transmition was aborted due to         ////
////  a collision.  A > is sent when reception begins to reduce      ////
////  the chance of attempting to transmit during reception.         ////
////                                                                 ////
////  Pin layout for the TW523:                                      ////
////                                                                 ////
////  SX Chip       TW523                                            ////
////  -------------------                                            ////
////  PIN_B0  --->  PIN 1                                            ////
////  PIN_B1  --->  PIN 3                                            ////
////  PIN_B2  --->  PIN 4                                            ////
////  GND     --->  PIN 2                                            ////
////                                                                 ////
/////////////////////////////////////////////////////////////////////////
////        (C) Copyright 1996,1997 Custom Computer Services            ////
//// This source code may only be used by licensed users of the CCS C   ////
//// compiler.  This source code may only be distributed to other       ////
//// licensed users of the CCS C compiler.  No other use, reproduction  ////
//// or distribution is permitted without written permission.           ////
//// Derivative programs created using this software in object code     ////
//// form are not restricted in any way.                                ////
////////////////////////////////////////////////////////////////////////////

#include <SX28AC.H>
#include "atd_emu.h"

#fuses HS,NOPROTECT,NOWDT

#use standard_io(B)
#use delay(clock=16000000)
#use rs232(baud=19200, xmit=PIN_A3, rcv=PIN_A2)

#include <x10.c>
#include <input.c>

main() {
   char house_code;
   byte key_code;

   set_mode_a(mTTLIN,mTTLIN,mTTLIN,mTTLIN,mOUTPUT,mTTLIN,mTTLIN,mTTLIN);
   set_mode_b(mTTLIN,mTTLIN,mTTLIN,mTTLIN,mTTLIN,mOUTPUT,mTTLIN|mPULLUP,mTTLIN);

   printf("Online\n\r");

   while (TRUE) {

      if(kbhit()) {
        house_code = getc();
        if((house_code>='A') && (house_code<='P')) {
          putc(house_code);
          key_code=gethex();
          x10_write(house_code,key_code);
          x10_write(house_code,key_code);
        }
      }

      if(x10_data_ready()) {
        putc('>');
        x10_read(&house_code,&key_code);
        printf("%c%2X",house_code,key_code);
      }
   }
}
