///////////////////////////////////////////////////////////////////////////
////                          SX_DPOT.C                                ////
////                                                                   ////
//// This example demonstrates how to use a digital pot.  The program  ////
//// will prompt for the pot settings, set the pot and show the        ////
//// results using an analog to digital converter.                     ////
////                                                                   ////
//// If you're using the AD8400 chip, the program will not prompt you  ////
//// to select a pot, since the chip only has one.                     ////
////                                                                   ////
////  SX Chip      DS1868                                             ////
////  -------------------                                            ////
////  PIN_B0  --->  PIN 5                                            ////
////  PIN_B1  --->  PIN 6                                            ////
////  PIN_B2  --->  PIN 8                                            ////
////  PIN_B3  --->  PIN 9                                            ////
////  GND     --->  PINS 1, 3, 7, 10                                 ////
////  +5      --->  PINS 2, 11, 14                                   ////
////  (5k between PIN 9 and 8 of DS1868)                             ////
////                                                                 ////
////  DS1868      LTC1298                                            ////
////  -------------------                                            ////
////  PIN 12  --->  PIN 2                                            ////
////  PIN 4   --->  PIN 3                                            ////
////                                                                 ////
////  SX Chip     LTC1298                                            ////
////  -------------------                                            ////
////  PIN_C0  --->  PIN 7                                            ////
////  PIN_C1  --->  PIN 6                                            ////
////  PIN_C2  --->  PIN 5                                            ////
////  PIN_C3  --->  PIN 1                                            ////
////  GND     --->  PIN 4                                            ////
////  +5      --->  PIN 8                                            ////
////                                                                 ////
////  Analog CH0 and CH1 are on pins 2 and 3 respectivly of the      ////
////  LTC1298                                                        ////
////                                                                 ////
///////////////////////////////////////////////////////////////////////////
////        (C) Copyright 1996,1997 Custom Computer Services            ////
//// This source code may only be used by licensed users of the CCS C   ////
//// compiler.  This source code may only be distributed to other       ////
//// licensed users of the CCS C compiler.  No other use, reproduction  ////
//// or distribution is permitted without written permission.           ////
//// Derivative programs created using this software in object code     ////
//// form are not restricted in any way.                                ////
////////////////////////////////////////////////////////////////////////////

#include <SX28AC.H>
#fuses   HS,NOPROTECT,NOWDT
#use delay(clock=16000000)

#define ADC_CLK  PIN_C0
#define ADC_DOUT PIN_C1
#define ADC_DIN  PIN_C2
#define ADC_CS   PIN_C3

#use rs232(baud=19200, xmit=PIN_A3, rcv=PIN_A2)

#include <INPUT.C>
#include <LTC1298.C>
#include <DS1868.C>

main() {
   long int   value;
   char  pot_num;
   set_mode_a(mTTLIN,mTTLIN,mTTLIN,mTTLIN,mOUTPUT,mTTLIN,mTTLIN,mTTLIN);
   set_mode_b(mOUTPUT,mOUTPUT,mOUTPUT,mOUTPUT,mOUTPUT,mOUTPUT,mOUTPUT,mOUTPUT);
   set_mode_c(mOUTPUT,mOUTPUT,mOUTPUT,mOUTPUT,mOUTPUT,mOUTPUT,mTTLIN,mOUTPUT);

   adc_init();

   init_pots ();

   do {
         value = read_analog(0);
      printf ("\n\r\n\rInput 1: %2LX  ", value);

      if (NUM_POTS > 1) {
         value = read_analog(1);
         printf ("Input 2: %2LX  \n\r", value);

         do {
            printf ("\n\rChange pot 1 or 2?");
            pot_num = getc ();
         } while ((pot_num!='1') && (pot_num!='2'));
         putc (pot_num);
      }
      else
         pot_num = '1';

      printf ("\n\rNew pot pot_value:");
      value = gethex ();

      set_pot ((pot_num-'1'), value);
   } while (TRUE);
}
